@@ -1,22 +1,15 @@
 import os
 import sys
-
 import torch
 
+# add root directory to path
+sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir)))
 from eeggan.helpers import system_inputs
 
-if __name__ == "__main__":
-
+def main():
     default_args = system_inputs.parse_arguments(sys.argv, kw_dict=system_inputs.default_inputs_get_gan_config())
 
-    file = default_args['file']
-
-    for arg in sys.argv:
-        if not arg.endswith('.py'):
-            if '=' in arg:
-                arg = arg.split('=')
-                if arg[0] == 'file':
-                    file = arg[1]
+    file = default_args['model']
 
     if file.split(os.path.sep)[0] == file:
         # use default path if no path is given
@@ -27,4 +20,7 @@ if __name__ == "__main__":
     print(f'\nGAN Configuration of {file}:')
     for key, value in state_dict['configuration'].items():
         print(f'\t{key}: {value}')
-    print('\n')
\ No newline at end of file
+    print('\n')
+
+if __name__ == "__main__":
+    main()
\ No newline at end of file
